<?php

/**
 *	Class Email
 *  -------------- 
 *  Description : encapsulates email operations & properties
 *  Updated	    : 26.07.2009
 *	Written by  : ApPHP
 *	Version     : 1.0.2   
 *	
 **/

  
define("EC_NEWLINE", "\r\n");                     /* newline character(s) */  
define("EC_XMAILER", "PHP-EMAIL-CLASS (ApPHP)");  /* the unique X-Mailer identifier */
define("EC_DEFAULTCHARSET", "iso-8859-1");        /* the default charset values text and HTML */
define("EC_UNIQEID", "MCMS_EMAIL");               /* unique ID */  												    
												    

class Email{
  
	// text email or a HTML email
	public $textOnly = true; 
	// the charset of the email
	public $charset = null; 

	private $to = null;  
	// (string) recipiant addresses to get a copy - can be a comma separated
	private $cc = null; 
	// (string) recipiant addresses to get a hidden copy - can be a comma separated
	private $bcc = null; 
	// (string) the email address of sender
	private $from = null; 
	// (string) the subject of the email
	private $subject = null; 
	// (string) body content for the email. Plain text or HTML based on the 'textOnly'
	public $content = null;
	// (array) email attachment instances
	public $arrAttachments;	
	// any header information that used when sending email
	private $headers = null; 
	
	
	//==========================================================================
	// Class Constructor
	//==========================================================================
	function __construct($to=null, $from=null, $subject=null, $headers=null) 
	{
		$this->to = $to; 
		$this->from = $from; 
		$this->subject = $subject; 
		$this->headers = $headers;   

		$this->arrAttachments = Array();    
		$this->arrAttachments["text"] = null;
		$this->arrAttachments["html"] = null;
	} 
	
	//==========================================================================
	// Class Destructor
	//==========================================================================
	function __destruct()
	{
		// echo 'this object has been destroyed';
	}	

	/**
	 * Check whether or not the email message is ready to be sent
	*/
	function IsComplete() 
	{ 
		return (strlen(trim($this->to)) > 0 && strlen(trim($this->from)) > 0); 
	} 

	/**
	 * Send the email message
	 * Returns: Boolean 
	*/
    function Send() 
    {
		// if message is not ready to send no message will be sent
		if(!$this->IsComplete()) return false;

		// get unique boundry identifier to separate attachments
		$the_boundary = "-----".md5(uniqid(EC_UNIQEID)); 		
		
		// -- HEADERS --- 
		// add from email address and the current date of sending
		$headers = "Date: ".date("r", time()).EC_NEWLINE."From: ".$this->from."".EC_NEWLINE;

		// add CC field to the headers, if not empty
		if(strlen(trim(strval($this->cc))) > 0) $headers .= "CC: ".$this->cc."".EC_NEWLINE;

		// add BCC field to the headers, if not empty
		if(strlen(trim(strval($this->bcc))) > 0) $headers .= "BCC: ".$this->bcc."".EC_NEWLINE;

		// add custom headers before important information 
		if($this->headers != null && strlen(trim($this->headers)) > 0) $headers .= $this->headers.EC_NEWLINE; 

		// is this email is mixed HTML and text or both
		$is_multipart_alternative = ($this->arrAttachments["text"] != null &&
								     $this->arrAttachments["html"] != null);

	    // set the correct MIME type
        $base_content_type = "multipart/".($is_multipart_alternative ? "alternative" : "mixed");

		// set headers 
		$headers .= "X-Mailer: ".EC_XMAILER.EC_NEWLINE. 
					"MIME-Version: 1.0".EC_NEWLINE. 
					"Content-Type: $base_content_type; ".
					"boundary=\"$the_boundary\"".EC_NEWLINE.EC_NEWLINE; 
	
		
		if($is_multipart_alternative){
			// add the text and html versions of the content
			// code here ...
		}else{
			// only html or text email	
			$the_email_type = "text/".($this->textOnly ? "plain" : "html"); 
		    if($this->charset == null) $this->charset = EC_DEFAULTCHARSET;
		
			// add the encoding header information to the body
			$the_body = "--$the_boundary".EC_NEWLINE. 
						"Content-Type: $the_email_type; charset=$this->charset".EC_NEWLINE.
						"Content-Transfer-Encoding: 8bit".EC_NEWLINE.EC_NEWLINE.
						$this->content.EC_NEWLINE.EC_NEWLINE; 		
		}
	
		// boundry marker
	    $the_body .= "--$the_boundary--";	

		// try to send the email message
	    return mail($this->to, $this->subject, $the_body, $headers); 
    } 
} 


?>