<?php

/**
 *	Class Languages
 *  -------------- 
 *  Description : encapsulates languages properties
 *  Updated	    : 25.11.2009
 *	Written by  : ApPHP
 *
 *	__construct
 *	__destruct
 *
 *	STATIC
 *	------------------
 *	GetDefaultLang
 *	LanguageExists
 *	
 *	PUBLIC
 *	------------------
 *	DrawLanguagesBar
 *	GetParameter
 *	DrawLanguages
 *	LanguageUpdate
 *	GetAll
 *	
 *	PRIVATE
 *	-----------------
 *	CopyDataToNewLang
 *	GetDataEncoded
 *	GetDataDecoded
 *	
 **/

class Languages {

	private $id;
	protected $language;
	public $error;    
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct($id = "") {
		$this->id = $id;
		if($this->id != ""){
			$sql = "SELECT
						id, lang_name, abbreviation, lang_dir, is_default, icon_image, lang_order, is_default,
						IF(is_default = 1, 'Yes', 'No') as is_default_verb
					FROM ".DB_PREFIX."languages WHERE id = '".intval($this->id)."'";
			$this->language = database_query($sql,_DATA_ONLY,_FIRST_ROW_ONLY);
		}else{
			$this->language['lang_name'] = "";
			$this->language['lang_order'] = "";
			$this->language['lang_dir'] = "";
			$this->language['abbreviation'] = "";
			$this->language['is_default'] = "";
			$this->language['icon_image'] = "";
		}
		
	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct(){
		// echo 'this object has been destroyed';
    }

	//==========================================================================
    // Static Methods
	//==========================================================================	
	/***
	 *	Get default language
	 **/
	static function GetDefaultLang(){
		$def_language = "en";
		$sql = "SELECT abbreviation FROM ".DB_PREFIX."languages WHERE is_default = 1";
		if($language = database_query($sql, _DATA_ONLY, _FIRST_ROW_ONLY)){
			$def_language = $language['abbreviation'];					
		}
		return $def_language;
	}

	/***
	 *	Check if language exists
	 **/
	static function LanguageExists($lang_abbrev){
		$sql = "SELECT abbreviation FROM ".DB_PREFIX."languages WHERE abbreviation = '".$lang_abbrev."'";
		if(database_query($sql, _ROWS_ONLY) > 0){
			return true;
		}
		return false;
	}

	/***
	 *	Get language direction
	 **/
	static function GetLanguageDirection($lang_abbrev = ""){
		$lang_dir = "ltr";
		$sql = "SELECT lang_dir FROM ".DB_PREFIX."languages WHERE abbreviation = '".$lang_abbrev."'";
		if($language = database_query($sql, _DATA_ONLY, _FIRST_ROW_ONLY)){
			$lang_dir = $language['lang_dir'];					
		}
		return $lang_dir;
	}

	//==========================================================================
    // Public Methods
	//==========================================================================

	/***
	 *	Draw languages bar
	 *	
	 **/
	public function DrawLanguagesBar($path = "page"){
		$all_languages = Languages::GetAll();
		if($all_languages > 0){
			for($i=0; $i < $all_languages[1]; $i++){
				echo "<a href='index.php?lang=".$all_languages[0][$i]['abbreviation']."' title='".$all_languages[0][$i]['lang_name']."'>".(file_exists("images/flags/".$all_languages[0][$i]['icon_image']) ? "<img src='images/flags/".$all_languages[0][$i]['icon_image']."' border='0' title='".$all_languages[0][$i]['lang_name']."' alt='".$all_languages[0][$i]['lang_name']."' />" : $all_languages[0][$i]['abbreviation'])."</a> ";
			}
		}
	}

	/***
	 *	Get language parameter
	 *	
	 **/
	public function GetParameter($param = ""){
		if(isset($this->language[$param])){
			return $this->language[$param];
		}else{
			return "";
		}
	}

	/***
	 *	Draw languages on the screen
	 *	
	 **/
	public function DrawLanguages(){
		global $text_alignment_left;
		#echo "<pre>";
		#print_r($this->language);
		#echo "</pre>";
		$all_languages = Languages::GetAll();
		
		if($all_languages > 0){
		
			echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='main_text'>
				<tr>
					<th align='".$text_alignment_left."'>".LANGUAGE_NAME."</th>
					<th width='12%'>".ICON_IMAGE."</th>
					<th width='12%'>".HDR_TEXT_DIRECTION."</th>
					<th width='12%'>".IS_DEFAULT."</th>
					<th width='12%'>".ORDER."</th>
					<th width='20%'>".ACTIONS."</th>
				</tr>
				<tr><td colspan='7' height='3px' nowrap='nowrap'>".draw_line("no_margin_line", _IMAGE_DIRECTORY, false)."</td></tr>";
		
				for($i=0; $i < $all_languages[1]; $i++){
					echo "<tr ".highlight(0)." onmouseover='oldColor=this.style.backgroundColor;this.style.backgroundColor=\"#e1e1e1\";' onmouseout='this.style.backgroundColor=oldColor'>
							<td align='".$text_alignment_left."'>".$all_languages[0][$i]['lang_name']." (".strtoupper($all_languages[0][$i]['abbreviation']).")</td>
							<td align='center'>".(file_exists("images/flags/".$all_languages[0][$i]['icon_image']) ? "<img src='images/flags/".$all_languages[0][$i]['icon_image']."' border='0' />" : "No Image")."</td>
							<td align='center'>".$all_languages[0][$i]['lang_dir']."</td>							
							<td align='center'>".$all_languages[0][$i]['is_default_verb']."</td>
							<td align='center'>".(int)$all_languages[0][$i]['lang_order']."</td>
							<td align='center' nowrap>
								<a href='index.php?admin=languages_edit&lid=".$all_languages[0][$i]['id']."' title='".CLICK_TO_EDIT."'>".EDIT_WORD."</a>
							</td>
						</tr>";
				}
			echo "</table>";
		}
	}

	/***
	 *	Edit language
	 *		@param $param - array of parameters
	 **/
	function LanguageUpdate($params = array()) {

		// Block operation in demo mode
		if(strtolower(_SITE_MODE) == "demo"){
			$this->error = OPERATION_BLOCKED;
			return false;
		}

		// Get input parameters
		if(!isset($params['id'])) 			$params['id'] = "0";
		if(!isset($params['lang_name'])) 	$params['lang_name'] = "";
		if(!isset($params['lang_dir']))     $params['lang_dir'] = "";
		if(!isset($params['lang_order'])) 	$params['lang_order'] = "";
		if(!isset($params['abbreviation'])) $params['abbreviation'] = "";
		if(!isset($params['icon_image'])) 	$params['icon_image'] = "";
		if(!isset($params['is_default'])) 	$params['is_default'] = "0";

		// Prevent creating of empty records in our 'languages' table
		if($params['lang_name'] == ""){
			$this->error = LANG_NAME_EMPTY;
			return false;
		}else if($params['abbreviation'] == ""){
			$this->error = LANG_ABBREV_EMPTY;
			return false;			
		}

		// check if such language already exists
		$sql = "SELECT *
				FROM ".DB_PREFIX."languages
				WHERE
					(lang_name = '".$params['lang_name']."' OR abbreviation = '".$params['abbreviation']."') AND
					id != ".(int)$params['id'];
		if(database_query($sql,_ROWS_ONLY) > 0){				
			$this->error = LANG_NAME_EXISTS;
			return false;			
		}
		
		$sql = "SELECT MIN(lang_order) as min_order, MAX(lang_order) as max_order FROM ".DB_PREFIX."languages";
		if($language = database_query($sql,_DATA_ONLY,_FIRST_ROW_ONLY)){
			$min_order = $language['min_order'];
			$max_order = $language['max_order'];	

			// insert language with new order priority in languages list
			$sql = "SELECT lang_order FROM ".DB_PREFIX."languages WHERE id = ".(int)$this->id;
			if($language = database_query($sql,_DATA_ONLY,_FIRST_ROW_ONLY)){
				$sql_down = "UPDATE ".DB_PREFIX."languages SET lang_order = lang_order - 1 WHERE id <> ".(int)$this->id." AND lang_order <= ".$params['lang_order']." AND lang_order > ".$language['lang_order'];
				$sql_up   = "UPDATE ".DB_PREFIX."languages SET lang_order = lang_order + 1 WHERE id <> ".(int)$this->id." AND lang_order >= ".$params['lang_order']." AND lang_order < ".$language['lang_order'];

				if($language['lang_order'] != $params['lang_order']){
					if($params['lang_order'] == $min_order){
						$sql = $sql_up;
					}else if($params['lang_order'] == $max_order){
						$sql = $sql_down;
					}else{
						if($language['lang_order'] < $params['lang_order']) $sql = $sql_down;
						else $sql = $sql_up;
					}
					$result = database_void_query($sql);
				}
				
				$sql = "UPDATE ".DB_PREFIX."languages SET
							lang_name = '".$this->GetDataEncoded($params['lang_name'])."',
							lang_dir  = '".$this->GetDataEncoded($params['lang_dir'])."',
							lang_order = '".$this->GetDataEncoded($params['lang_order'])."', 
							abbreviation = '".$this->GetDataEncoded($params['abbreviation'])."',
							icon_image = '".$this->GetDataEncoded($params['icon_image'])."'
						WHERE id = ".(int)$this->id;
				$result = (database_void_query($sql) >= 0) ? true : false;
				
				// set defaulr language
                if($params['is_default'] == "1"){					
					$sql = "UPDATE ".DB_PREFIX."languages SET is_default = IF(id = ".(int)$this->id.", 1, 0)";
					$result = (database_void_query($sql) >= 0) ? true : false;
				}
				
			}
		}
	
		if($result){
			return true;
		}else{
			$this->error = TRY_LATER;
			return false;
		}				
	}

	/***
	 *	Get all languages array
	 *		@param $order - order clause
	 *		@param $join_table - join tables
	 **/
	function GetAll($order = " lang_order ASC", $join_table = "") {
		$where_clause = "";
		
		// Build ORDER BY CLAUSE
		if($order=="")$order_clause = "";
		else $order_clause = "ORDER BY $order";		

		// Build JOIN clause
		$join_clause = "";
		$join_select_fields = "";
		
		$sql = "SELECT
					id, lang_name, abbreviation, lang_dir, is_default, icon_image, lang_order, is_default,
					IF(is_default = 1, 'Yes', 'No') as is_default_verb
				FROM ".DB_PREFIX."languages 
					$join_clause
				$order_clause";			
		
		return database_query($sql,_DATA_AND_ROWS);
	}


	//==========================================================================
    // Private Methods
	//==========================================================================
	
	/***
	 *	Copy default data for new language
	 *		@param $string
	 **/
	private function CopyDataToNewLang($from_abbrev, $to_abbrev){
		
		// clone data for Menus table
		$sql = "INSERT INTO ".DB_PREFIX."menus (language_id, menu_name, menu_order)
				(SELECT '".$to_abbrev."', menu_name, menu_order FROM ".DB_PREFIX."menus WHERE language_id = '".$from_abbrev."')";
		database_void_query($sql);					

		// clone data for Vocabulary table
		$sql = "INSERT INTO ".DB_PREFIX."vocabulary (language_id, key_value, key_text)
				(SELECT '".$to_abbrev."', key_value, key_text FROM ".DB_PREFIX."vocabulary WHERE language_id = '".$from_abbrev."')";
		database_void_query($sql);					

		// clone data for Static Pages table
		$sql = "INSERT INTO ".DB_PREFIX."static_pages (language_id, page_key, page_title, page_text, menu_id, menu_link, menu_placement, is_home)
				(SELECT '".$to_abbrev."', page_key, page_title, page_text, menu_id, menu_link, menu_placement, is_home FROM ".DB_PREFIX."static_pages WHERE language_id = '".$from_abbrev."')";
		database_void_query($sql);					

		// copy default messages.inc.php for new language
		$source_file = "include/messages.".$from_abbrev.".inc.php";
		$desfination_file = "include/messages.".$to_abbrev.".inc.php";
		@copy($source_file, $desfination_file);
	}
	

	/***
	 *	Return encoded data
	 *		@param $string
	 **/
	private function GetDataEncoded($string = ""){
		return $string;
	}
	
	/***
	 *	Return decoded data
	 *		@param $string
	 **/
	private function GetDataDecoded($string = ""){
		$string = str_replace('\\\\', "\\", $string); // double quotation mark
		$string = str_replace('\"', "&#034;", $string); // double quotation mark
		$string = str_replace("\'", "&#039;", $string); // single quotation mark                        
		$string = str_replace('"', "&#034;", $string); // double quotation mark
		$string = str_replace("'", "&#039;", $string); // single quotation mark                        
		return $string;
	}
}
?>