<?php

/**
 *	Class Login
 *  -------------- 
 *  Description : encapsulates login properties
 *  Updated     : 26.11.2009	
 *	Written by  : ApPHP
 *	
 **/

class Login {

	private $wrong_login;
	private $messages;
	private $accounts;
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct()
	{
		$submit_login  = isset($_POST['submit_login']) ? remove_bad_chars($_POST['submit_login']) : "";
		$submit_logout = isset($_POST['submit_logout']) ? remove_bad_chars($_POST['submit_logout']) : "";
		$user_name     = isset($_POST['user_name']) ? prepare_input($_POST['user_name'], true) : "";
		$password      = isset($_POST['password']) ? prepare_input($_POST['password'], true) : "";
		$this->wrong_login = false;		

		if (!$this->IsLoggedIn()){
			if($submit_login == "Login"){
				if(empty($user_name) || empty($password)){
					$this->wrong_login = true;							
				}else{
					$this->DoLogin($user_name, $password);
				}
			}			
		}else if ($submit_logout == "Logout"){
			$this->DoLogout();
		}
		$this->accounts = new Accounts($GLOBALS['user_session']->GetSessionVariable("session_account_id"));
	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct()
	{
		// echo 'this object has been destroyed';
    }

	/***
	 * 	Do login
	 * 		@param $user_name - system name of user
	 * 		@param $password - password of user
	 * 		@param $do_redirect - prepare redirect or not
	 **/
	private function DoLogin($user_name, $password, $do_redirect = true)
	{
		if ($account_information = $this->GetAccountInformation($user_name, $password)) {
			$this->SetSessionVariables($account_information);
			$this->UpdateAccountInfo($account_information);
			if ($do_redirect) {
				header("Location: index.php");
				exit;
			}
		}else{
			$this->wrong_login = true;
		}
	}
	
	/***
	 * 	Return id login was wrong
	 * 	
	 **/
	function IsWrongLogin()
	{
		return ($this->wrong_login == true) ? true : false;	
	}

	/***
	 * 	Destroys the session and returns to the default page
	 * 	
	 **/
	function DoLogout()
	{
		$GLOBALS['user_session']->EndSession();
	}

	/***
	 * 	Gets the account information
	 * 		@param $user_name - system name of user
	 * 		@param $password - password of user
	 **/
	private function GetAccountInformation($user_name, $password)
	{
		if(_PASSWORD_ENCRYPTION){
			$password = "AES_ENCRYPT('".$password."', '".DB_ENCRYPT_KEY."')";
		}else{
			$password = "'".$password."'";
		}
		$sql = "SELECT ".DB_PREFIX."accounts.*, user_name AS account_name
				FROM ".DB_PREFIX."accounts
				WHERE user_name = '" . $user_name . "' AND 
					  password = ".$password;
		return database_query($sql,_DATA_ONLY,_FIRST_ROW_ONLY);
	}

	/***
	 * 	Checks to see if the user is logged in
	 * 		@return a 1 if the user is logged in, 0 otherwise
	 **/
	public function IsLoggedIn()
	{
		$logged = $GLOBALS['user_session']->GetSessionVariable('session_account_id');
		return !empty($logged);
	}

	/***
	 * 	Get last login time
	 * 		@return last login time
	 **/
	function GetLastLoginTime()
	{
		$last_login = $GLOBALS['user_session']->GetSessionVariable("session_last_login");

		if($last_login != "" && $last_login != "0000-00-00 00:00:00" && $last_login != "NULL"){
			return $last_login;
		}else{
			return "--";
		}
	}

	/***
	 * 	Checks to see if the user is logged in as a specific account type
	 * 		@return true if the user is logged in as specified account type,
	 * 			    false otherwise
	 **/
	function IsLoggedInAs()
	{
		if (!$this->IsLoggedIn()) return false;

		$account_type = $GLOBALS['user_session']->GetSessionVariable("session_account_type");

		$types = func_get_args();
		foreach ($types as $type) {
			if ($account_type == $type) return true;
		}

		return false;
	}

	/***
	 * 	Sets the session variables and performs the login
	 * 		@param $account_information - array
	 **/
	function SetSessionVariables($account_information)
	{
		$GLOBALS['user_session']->SetSessionVariable("session_account_id", $account_information['id']);
		$GLOBALS['user_session']->SetSessionVariable("session_user_name", $account_information['user_name']);
		$GLOBALS['user_session']->SetSessionVariable("session_account_type", $account_information['account_type']);
		$GLOBALS['user_session']->SetSessionVariable("session_last_login", @date("Y-m-d H:i:s"));		
	}
	
	/***
	 * 	Update Account Info	
	 * 		@param $account_information - array
	 **/
	function UpdateAccountInfo($account_information)
	{
		$sql = "UPDATE ".DB_PREFIX."accounts
				SET last_login = '".@date("Y-m-d H:i:s")."'
				WHERE id = ".(int)$account_information['id']."";
		return database_void_query($sql);
	}	

	/***
	 * 	Draws the login welcome message
	 * 	
	 **/	
	function DrawWelcomeMessage()
	{
		if($GLOBALS['user_session']->GetSessionVariable("session_user_name") != "")
			echo "<b>".ucfirst($GLOBALS['user_session']->GetSessionVariable("session_user_name"))." ".WELCOME_ADMIN."</b>";
	}

	/***
	 * 	Draws the login links and logout form
	 * 	
	 **/
	function DrawLoginLinks()
	{
		global $objSettings;
		
		$text_align = ($objSettings->GetDirection() == "ltr") ? "text-align: left;" : "text-align: right; padding-right:15px;";
		
		// Main Admin Links
		if ($this->IsLoggedIn()) {			
			draw_block_top();
				echo "<table class='moduletable' width='100%' border='0' cellspacing='0' cellpadding='0'>";
				echo "<tr><th valign='top' style='".$text_align."'><strong class='title_link'>".GENERAL."</strong></th></tr>";
				echo "<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php'>".HOME."</a></td></tr>";
				if ($this->IsLoggedInAs('mainadmin')) echo "<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?admin=settings'>".SETTINGS."</a></td></tr>";
				if ($this->IsLoggedInAs('mainadmin')) echo "<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?admin=modules'>".MODULES."</a></td></tr>";
				echo "<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?preview=1' target='_blank'>".SITE_PREVIEW."</a></td></tr>";
				echo "</table>";
			draw_block_bottom();
		}
			
		if ($this->IsLoggedIn()) {
			draw_block_top();
				echo "<table class='moduletable' width='100%' border='0' cellspacing='0' cellpadding='0'>";
				echo "<tr><th valign='top' style='".$text_align."'><strong class='title_link'>".ACCOUNTS_MANAGEMENT."</strong></th></tr>";
				echo "<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?admin=my_account'>".MY_ACCOUNT."</a></td></tr>";
				if ($this->IsLoggedInAs('mainadmin')) echo "<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?admin=admins_management'>".ADMINS."</a></td></tr>";
				echo "</table>";
			draw_block_bottom();
		}

		if ($this->IsLoggedIn()) {
			draw_block_top();
				echo "<table class='moduletable' width='100%' border='0' cellspacing='0' cellpadding='0'>";
				echo "<tr><th valign='top' style='".$text_align."'><strong class='title_link'>".LANGUAGES_SETTINGS."</strong></th></tr>";
				if ($this->IsLoggedInAs('mainadmin')) echo "<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?admin=languages'>".LANGUAGES."</a></td></tr>";
				echo "<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?admin=vocabulary&filter_by=A'>".VOCABULARY."</a></td></tr>";
				echo "</table>";
			draw_block_bottom();
		}

		if ($this->IsLoggedInAs('mainadmin')) {
			draw_block_top();		
				echo "<table class='moduletable' width='100%' border=0 cellspacing=0 cellpadding=0>
				<tr><th valign='top' style='".$text_align."'><strong class='title_link'>".MENU_MANAGEMENT."</strong></th></tr>
				<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?admin=menus_add'>".ADD_NEW_MENU."</a></td></tr>
				<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?admin=menus'>".EDIT_MENUS."</a></td></tr>
				</table>";
			draw_block_bottom();
		}

		if ($this->IsLoggedIn()) {
			draw_block_top();
				echo "<table class='moduletable' width='100%' border=0 cellspacing=0 cellpadding=0>
				<tr><th valign='top' style='".$text_align."'><strong class='title_link'>".PAGE_MANAGEMENT."</strong></th></tr>
				<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?admin=static_pages_add'>".PAGE_ADD_NEW."</a></td></tr>
				<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?admin=static_pages_edit'>".PAGE_EDIT_HOME."</a></td></tr>
				<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?admin=static_pages'>".PAGE_EDIT_PAGES."</a></td></tr>				
				</table>";
			draw_block_bottom();
		}
		
		if ($this->IsLoggedIn()) {
			// MODULES
			$sql = "SELECT * FROM ".DB_PREFIX."modules WHERE is_installed = 1";
			$modules = database_query($sql,_DATA_AND_ROWS,_ALL_ROWS);			
			for($i=0; $i < $modules[1]; $i++){				
				if($modules[0][$i]['name'] == "backup"){
					draw_block_top();
						echo "<table class='moduletable' width='100%' border='0' cellspacing='0' cellpadding='0'>";
						echo "<tr><th valign='top' style='".$text_align."'><strong class='title_link'>".BACKUP_AND_RESTORE."</strong></th></tr>";
						echo "<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?admin=backup_installation'>".BACKUP_INSTALLATION."</a></td></tr>";
						if ($this->IsLoggedInAs('mainadmin')) echo "<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?admin=backup_restore'>".BACKUP_RESTORE."</a></td></tr>";
						echo "</table>";
					draw_block_bottom();						
				}				
			}						
		}

		// Logout
		if ($this->IsLoggedIn()){
            echo "<table class='moduletable' width='100%' border=0 cellspacing=0 cellpadding=0>
                  <tr><td height='5px' nowrap></td></tr>
                  <tr><td><form action='index.php' method='post'>";
                  draw_token_field();
            echo "&nbsp;&nbsp;<input type='hidden' name='submit_logout' value='Logout' />";
            echo "<input class='form_button' type='submit' name='btnLogout' value='".BUTTON_LOGOUT."' />&nbsp;&nbsp;</form></td>
                  </tr></table>";
		}
	}
}
?>