<?php

/**
 *	Class Menu
 *  -------------- 
 *  Description : encapsulates menu properties
 *  Updated	    : 16.05.2009
 *	Written by  : ApPHP
 *	
 **/

class Menu {

	private $id;
	
	protected $menu;
	protected $languageId;
	protected $whereClause;
	
	public $langIdByUrl;
	public $error;    
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct($id = "") {
		$this->id = $id;
		$this->languageId  = (isset($_REQUEST['language_id']) && $_REQUEST['language_id'] != "") ? $_REQUEST['language_id'] : Languages::GetDefaultLang();
		$this->whereClause  = "";
		$this->whereClause .= ($this->languageId != "") ? " AND language_id = '".$this->languageId."'" : "";		
		$this->langIdByUrl = ($this->languageId != "") ? "&language_id=".$this->languageId : "";
		
		if($this->id!=""){
			$sql = "SELECT
						".DB_PREFIX."menus.*,
						".DB_PREFIX."languages.lang_name as language_name
					FROM ".DB_PREFIX."menus
						LEFT OUTER JOIN ".DB_PREFIX."languages ON ".DB_PREFIX."menus.language_id = ".DB_PREFIX."languages.abbreviation
					WHERE ".DB_PREFIX."menus.id = '".intval($this->id)."'";
			$this->menu = database_query($sql,_DATA_ONLY,_FIRST_ROW_ONLY);
		}else{
			$this->menu['menu_name'] = "";
			$this->menu['menu_order'] = "";
			$this->menu['language_id'] = "";
			$this->menu['language_name'] = "";
		}
	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct(){
		// echo 'this object has been destroyed';
    }

	/***
	 *	Get menu name
	 *	
	 **/
	public function GetName(){		
		if(isset($this->menu['menu_name'])) return $this->GetDataDecoded($this->menu['menu_name']);
		else return "";
	}

	/***
	 *	Get parameter
	 *	
	 **/
	public function GetParameter($param = ""){
		if(isset($this->menu[$param])){
			return $this->menu[$param];
		}else{
			return "";
		}
	}
	
	/***
	 *	Get menu ID
	 *	
	 **/
	public function GetId(){
		return $this->id;
	}
	
	/***
	 *	Get menu order
	 *	
	 **/
	public function GetOrder(){
		if(isset($this->menu['menu_order'])) return $this->menu['menu_order'];
		else return "";
	}
	
	/***
	 *	Get all top menus array
	 *	
	 **/
	function GetAllTop($where_clause = "", $lang_id = ""){
		if($lang_id != "") $where_clause .= "AND ".DB_PREFIX."static_pages.language_id = '".$lang_id."' ";
		$where_lang = '';
		// Get all top menus
		$sql = "SELECT ".DB_PREFIX."static_pages.* 
				FROM ".DB_PREFIX."static_pages 
				WHERE ".DB_PREFIX."static_pages.menu_placement = 'top' ".$where_clause." ".$where_lang;
		return database_query($sql,_DATA_AND_ROWS);
	}

	/***
	 *	Get all left menus array
	 *	
	 **/
	function GetLeftMenus($lang_id = ""){
		// Get all left menus
		$sql = "SELECT
					".DB_PREFIX."menus.*
				FROM ".DB_PREFIX."menus 
				    INNER JOIN ".DB_PREFIX."languages ON ".DB_PREFIX."menus.language_id = ".DB_PREFIX."languages.abbreviation
				WHERE
					".DB_PREFIX."menus.language_id = '".$lang_id."'
				ORDER BY ".DB_PREFIX."menus.menu_order ASC ";
		return database_query($sql,_DATA_AND_ROWS);
	}
	
	/***
	 *	Get all left menu links array
	 *	
	 **/
	function GetLeftMenuLinks($menu_id, $lang_id = ""){
		// Get all left menus
		$sql = "SELECT
					".DB_PREFIX."static_pages.*
				FROM ".DB_PREFIX."static_pages
				    INNER JOIN ".DB_PREFIX."languages ON ".DB_PREFIX."static_pages.language_id = ".DB_PREFIX."languages.abbreviation
				WHERE
					".DB_PREFIX."static_pages.language_id = '".$lang_id."' AND
					".DB_PREFIX."static_pages.menu_placement = 'left' AND
					".DB_PREFIX."static_pages.menu_id = '".$menu_id."' AND
					".DB_PREFIX."static_pages.is_home = 0
				ORDER BY ".DB_PREFIX."static_pages.id ASC ";
		return database_query($sql,_DATA_AND_ROWS);
	}

	/***
	 *	Get all menus array
	 *		@param $order - order clause
	 *		@param $join_table - join tables
	 **/
	function GetAll($order = " menu_order ASC", $join_table = "", $menu_placement = "", $lang_id = "") {
		$where_clause = "";
		if($menu_placement == "left"){
			$where_clause .= "AND ".DB_PREFIX."static_pages.menu_placement = 'left' ";
		}else if($menu_placement == "top"){
			$where_clause .= "AND ".DB_PREFIX."static_pages.menu_placement = 'top' ";
		}
		if($lang_id != "") $where_clause .= "AND ".DB_PREFIX.$join_table.".language_id = '".$lang_id."' ";
		
		// Build ORDER BY CLAUSE
		if($order=="")$order_clause = "";
		else $order_clause = "ORDER BY $order";		

		$join_clause = "";
		$join_select_fields = "";

		// Build JOIN clause
		if($join_table == "") {
			$join_clause = "";
			$join_select_fields = "";
		}else if($join_table != "menus") {
			$join_clause = "LEFT OUTER  JOIN ".DB_PREFIX.$join_table." ON ".DB_PREFIX.$join_table.".menu_id=".DB_PREFIX."menus.id ";
			$join_select_fields = ", ".DB_PREFIX.$join_table.".* ";
		}
		
		$sql = "SELECT
					".DB_PREFIX."menus.*,
					".DB_PREFIX."languages.lang_name as language_name
					$join_select_fields 
				FROM ".DB_PREFIX."menus 
				    INNER JOIN ".DB_PREFIX."languages ON ".DB_PREFIX."menus.language_id = ".DB_PREFIX."languages.abbreviation
					$join_clause
				WHERE 1=1
				$where_clause
				$order_clause";			
		
		return database_query($sql,_DATA_AND_ROWS, _ALL_ROWS, _FETCH_ASSOC);
	}
	
	
	/***
	 *	Update menu 
	 *		@param $param - array of parameters
	 **/
	function MenuUpdate($params = array()) {

		// Block operation in demo mode
		if(strtolower(_SITE_MODE) == "demo"){
			$this->error = OPERATION_BLOCKED;
			return false;
		}

		if(isset($this->id)){
			// Get input parameters
			if(isset($params['name']) && $params['name']!=""){
				$this->menu['menu_name'] = $params['name'];
			}else{
				$this->error = MENU_NAME_EMPTY;
				return false;
			}
			if(isset($params['order'])) $this->menu['menu_order'] = $params['order'];
		    if(isset($params['language_id'])) $this->menu['language_id'] = $params['language_id'];
			
			$sql = "SELECT MIN(menu_order) as min_order, MAX(menu_order) as max_order FROM ".DB_PREFIX."menus";
			if($menu = database_query($sql,_DATA_ONLY,_FIRST_ROW_ONLY)){
				$min_order = $menu['min_order'];
				$max_order = $menu['max_order'];
				
				// insert menu with new order priority in menus list
				$sql = "SELECT menu_order FROM ".DB_PREFIX."menus WHERE id = ".(int)$this->id;
				if($menu = database_query($sql,_DATA_ONLY,_FIRST_ROW_ONLY)){
					$sql_down = "UPDATE ".DB_PREFIX."menus SET menu_order = menu_order - 1 WHERE language_id = '".$this->menu['language_id']."' AND id <> ".(int)$this->id." AND menu_order <= ".$this->menu['menu_order']." AND menu_order > ".$menu['menu_order'];
					$sql_up = "UPDATE ".DB_PREFIX."menus SET menu_order = menu_order + 1 WHERE language_id = '".$this->menu['language_id']."' AND id <> ".(int)$this->id." AND menu_order >= ".$this->menu['menu_order']." AND menu_order < ".$menu['menu_order'];
					
					if($menu['menu_order'] != $this->menu['menu_order']){							
						$sql = "UPDATE ".DB_PREFIX."menus
						        SET
								    language_id = '".$this->menu['language_id']."',
									menu_name = '".$this->GetDataEncoded($this->menu['menu_name'])."',								
								    menu_order = ".$this->menu['menu_order']."
								WHERE id = ".(int)$this->id." AND menu_order <> ".$this->menu['menu_order'];
						if($result = database_void_query($sql)){
							if($this->menu['menu_order'] == $min_order){
								$sql = $sql_up;
							}else if($this->menu['menu_order'] == $max_order){
								$sql = $sql_down;
							}else{
								if($menu['menu_order'] < $this->menu['menu_order']) $sql = $sql_down;
								else $sql = $sql_up;
							}
							$result = database_void_query($sql);
						}							
					}else{
						$sql = "UPDATE ".DB_PREFIX."menus
						        SET
									language_id = '".$this->menu['language_id']."',
								    menu_name = '".$this->GetDataEncoded($this->menu['menu_name'])."'
								WHERE id = ".(int)$this->id;
						$result = database_void_query($sql);
					}
				}
			}

			if($result >= 0){
				return true;
			}else{
				$this->error = TRY_LATER;
				return false;
			}				
		}else{
			$this->error = MENU_MISSED;
			return false;
		}
	}
	
	/***
	 *	Create new menu 
	 *		@param $param - array of parameters
	 **/
	function MenuCreate($params = array()) {

		// Block operation in demo mode
		if(strtolower(_SITE_MODE) == "demo"){
			$this->error = OPERATION_BLOCKED;
			return false;
		}

		// Get input parameters
		if(isset($params['name'])) $this->menu['menu_name'] = $params['name'];
		if(isset($params['order'])) $this->menu['menu_order'] = $params['order'];
		if(isset($params['language_id'])) $this->menu['language_id'] = $params['language_id'];

		// Prevent creating of empty records in our 'menus' table
		if($this->menu['menu_name']!=""){
			$menu_code = strtoupper(random_string(10));

			$total_languages = Languages::GetAll();
			for($i = 0; $i < $total_languages[1]; $i++){				

				$m = Menu::GetAll(" menu_order ASC", "menus", "", $total_languages[0][$i]['abbreviation']);
				$max_order = (int)($m[1]+1);			

				$sql = "INSERT INTO ".DB_PREFIX."menus (language_id, menu_code, menu_name, menu_order)
						VALUES('".$total_languages[0][$i]['abbreviation']."', '".$menu_code."', '".$this->GetDataEncoded($this->menu['menu_name'])."', ".$max_order.")";
				if(!database_void_query($sql)){
					$this->error = TRY_LATER;
					return false;
				}
				//else{					
					//$this->menu['language_id'] = $params['language_id'] = $total_languages[0][$i]['abbreviation'];					
					//$sql = "SELECT * FROM ".DB_PREFIX."menus WHERE menu_order = ".$max_order;                    
					//if($menu = database_query($sql,_DATA_ONLY,_FIRST_ROW_ONLY)){                        
					   //$this->id = $menu['id'];
					   //$this->MenuUpdate($params);
					   //print_r($params);
					//}                    
				//}				
			}
			return true;			
		}else{
			$this->error = MENU_NAME_EMPTY;
			return false;
		}
	}

	/***
	 *	Delete menu 
	 *		@param $menu_id - menu ID
	 **/
	function MenuDelete($menu_id = "0", $menu_order = "0") {

		// Block operation in demo mode
		if(strtolower(_SITE_MODE) == "demo"){
			$this->error = OPERATION_BLOCKED;
			return false;
		}

		$sql = "DELETE FROM ".DB_PREFIX."menus WHERE id = ".(int)$menu_id."";
		if(database_void_query($sql)){
			$sql = "UPDATE ".DB_PREFIX."menus SET menu_order = menu_order - 1 WHERE menu_order > ".(int)$menu_order."";
			if(database_void_query($sql)){
				return true;    
			}                				   
		}

		return false;
	}

	/***
	 *	Move menu (change priority order)
	 *		@param $menu_id
	 *		@param $dir - direction
	 *		@param $menu_order  - menu order
	 **/
	function MenuMove($menu_id, $dir = "", $menu_order = "") {		

		// Block operation in demo mode
		if(strtolower(_SITE_MODE) == "demo"){ 
			$this->error = OPERATION_BLOCKED;
			return false;
		}

		if(($dir == "") || ($menu_order == "")) return false;		

		$sql = "SELECT * FROM ".DB_PREFIX."menus
				WHERE
					id <> '".(int)$menu_id."' AND
					menu_order ".(($dir == "up") ? "<" : ">")." ".(int)$menu_order." AND
					language_id = '".$this->languageId."'
				ORDER BY menu_order ".(($dir == "up") ? "DESC" : "ASC");
        if($menu = database_query($sql,_DATA_ONLY,_FIRST_ROW_ONLY)){
			$sql = "UPDATE ".DB_PREFIX."menus SET menu_order = '".$menu_order."' WHERE id = ".(int)$menu['id']."";
			if(database_void_query($sql)){
				$sql = "UPDATE ".DB_PREFIX."menus SET menu_order = '".$menu['menu_order']."' WHERE id = ".(int)$menu_id."";				
				if(!database_void_query($sql)){
					$this->error = TRY_LATER;
					return false;					
				}
			}else{
				$this->error = TRY_LATER;
				return false;
			}
		}
		return true;		
	}

	/***
	 *	Draw all menus in dropdowm box
	 *		@param $menu_id
	 **/
	function DrawMenuSelectBox($menu_id = "", $menu_placement = "left", $language_id = "") {
		$disabled = ($menu_placement == "top") ? "disabled='disabled'" : ""; 
		
		echo "<select name='menu_id' id='menu_id' ".$disabled.">";
		echo "<option value=''>-- select --</option>";
		$all_menus = self::GetAll(" menu_order ASC", "menus", "", $language_id);		                 
		for($i = 0; $i < $all_menus[1]; $i++){
			echo "<option value='".$all_menus[0][$i]['id']."'";
			echo ($all_menus[0][$i]['id'] == $menu_id) ? " selected " : "";
			echo ">".$all_menus[0][$i]['menu_name']."</option>";
		}
		echo "</select>";		
	}

	/***
	 *	Draw menus placement in dropdowm box
	 *		@param $menu_id
	 **/
	function DrawMenuPlacementBox($menu_placement = "") {
		echo "<select name='menu_placement' onchange=\"MenuPlacement_OnChange(this.value);\" >";
		echo "<option value='left' ".(($menu_placement == "left") ? " selected " : "").">".LEFT."</option>";
		echo "<option value='top' ".(($menu_placement == "top") ? " selected " : "").">".TOP."</option>";
		echo "</select>";		
	}

	/***
	 *	Draw all menus as a tree (for left menu)
	 *		@param $is_logged_in
	 *		@param $site_preview
	 **/
	function DrawTopMenu($is_logged_in = false, $site_preview = "0") {
		// If not logged in or site preview == 1 - draw menu
		global $objSettings;

		if(!$is_logged_in || $site_preview){
			echo "<a class='main_link' href='index.php'>".HOME."</a>";
		
			$lang = (isset($_SESSION['lang'])) ? $_SESSION['lang'] : Languages::GetDefaultLang();
			$menus = self::GetAllTop("", $lang);
			for($menu_ind = 0; $menu_ind < $menus[1]; $menu_ind++) {
				// Draw current menu link
				echo "&nbsp;".get_divider()."&nbsp;";
                echo "<a class='main_menu_link' href='index.php?page=static_pages&pid=".$menus[0][$menu_ind]['id'].(($site_preview == "1") ? "&preview=1" : "")."'>".camel_case($menus[0][$menu_ind]['menu_link'])."</a>";
			}
		}else{
			echo "<a class='main_link' href='index.php?admin=home'>".HOME."</a>";	
		}
	}

	/***
	 *	Draw all menus as a tree (for left menu)
	 *		@param $is_logged_in
	 *		@param $site_preview
	 **/
	function DrawTree($is_logged_in = false, $site_preview = "0") {
		global $objSettings;
		
		// If not logged in or site preview == 1 - draw menu		
		if($is_logged_in && $site_preview != "1"){
			return false;
		}

		// Get all menus which have items (links to static pages)
		$lang = (isset($_SESSION['lang'])) ? $_SESSION['lang'] : Languages::GetDefaultLang();
		$menus = self::GetLeftMenus($lang);
	    $text_align = ($objSettings->GetDirection() == "ltr") ? "text-align: left;" : "text-align: right; padding-right:15px;";

		// Display all menu titles (names) according to their order
		$cur_menu = "";
		$is_start = true;
		for($menu_ind = 0; $menu_ind < $menus[1]; $menu_ind++) {				
				
			// Start draw new menu
			draw_block_top();
			echo "<table class='moduletable' width='100%' border='0' cellspacing='0' cellpadding='0'>
				  <tr><th valign='top' style='".$text_align."'>".camel_case($menus[0][$menu_ind]['menu_name'])."</th></tr>";

			$menu_links = self::GetLeftMenuLinks($menus[0][$menu_ind]['id'], $lang);		
			for($menu_link_ind = 0; $menu_link_ind < $menu_links[1]; $menu_link_ind++) {
				// Draw current menu link
				echo "<tr><td>&nbsp;&nbsp;&nbsp;&#8226;&nbsp;<a class='main_menu_link' href='index.php?page=static_pages&pid=".$menu_links[0][$menu_link_ind]['id'].(($site_preview == "1") ? "&preview=1" : "")."'>".camel_case($menu_links[0][$menu_link_ind]['menu_link'])."</a></td></tr>";
			}
			echo "</table>";
			draw_block_bottom();
        }
		
		echo "<table class='moduletable' width='100%' border='0' cellspacing='0' cellpadding='0'>";
		echo "<tr><td>&nbsp;</td></tr>
		     <tr>
				<td align='left'>&nbsp;&nbsp;";
					echo "<a href='mailto:".$objSettings->GetParameter("admin_email")."' title='".CONTACT_US."'><img src='images/letter.png' alt='email' border='0' /></a>
				</td>
			 </tr>";			
		echo "</table>";			

		
		// End drawing last menu
		if(!$is_start){
            echo "</table>";
            draw_block_bottom();
        }
	}

	/***
	 *	Return encoded data
	 *		@param $string
	 **/
	private function GetDataEncoded($string = ""){
		return $string;
	}
	
	/***
	 *	Return decoded data
	 *		@param $string
	 **/
	private function GetDataDecoded($string = ""){
		$string = str_replace('\\\\', "\\", $string); // double quotation mark
		$string = str_replace('\"', "&#034;", $string); // double quotation mark
		$string = str_replace("\'", "&#039;", $string); // single quotation mark                        
		$string = str_replace('"', "&#034;", $string); // double quotation mark
		$string = str_replace("'", "&#039;", $string); // single quotation mark                        
		return $string;
	}
}
?>