<?php

/**
 *	Class MicroGrid
 *  -------------- 
 *  Description : encapsulates grid operations & properties
 *  Updated	    : 01.12.2009
 *	Written by  : ApPHP
 *	Version     : 1.0.2
 *	
 **/

class MicroGrid {
		
	private $debug = false;
		
	protected $tableName;
	protected $primaryKey;
	protected $dataSet;
	
	protected $formActionURL;
	protected $params;
	protected $actions;

	protected $languageId;
	protected $allowLanguages;
	
	protected $VIEW_MODE_SQL;
	protected $EDIT_MODE_SQL;	
	protected $DETAILS_MODE_SQL;
	protected $WHERE_CLAUSE;
	protected $ORDER_CLAUSE;
	protected $LIMIT_CLAUSE;
	
	protected $arrViewModeFields;	
	protected $arrAddModeFields;	
	protected $arrEditModeFields;
	protected $arrDetailsModeFields;
	
	protected $isHtmlEncoding;
	
	//TODO
	// - add try catch + error
	// returns errors
	
	public $error;
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct($id = "") {

		$this->params = array();

		$this->isHtmlEncoding = false;
		
		$this->primaryKey       = "";
		$this->tableName        = "";
		$this->formActionURL    = "";
		
		$this->languageId  	    = "";
		$this->allowLanguages   = false;

		$this->VIEW_MODE_SQL    = "";
		$this->EDIT_MODE_SQL    = "";
		$this->DETAILS_MODE_SQL = "";
		$this->WHERE_CLAUSE     = "";
		$this->ORDER_CLAUSE     = "";	
		$this->LIMIT_CLAUSE     = "";
		
		$this->arrViewModeFields    = array();
		$this->arrAddModeFields     = array();
		$this->arrEditModeFields    = array();
		$this->arrDetailsModeFields = array();
		
		$this->actions = array("add"=>true, "edit"=>true, "details"=>true, "delete"=>true);
		
		$this->dataSet = array();
		$this->error = "";		
	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct()
	{
		// echo 'this object has been destroyed';
    }


	/***
	 *	Get all DataSet array
	 **/
	function GetAll($order_clause = "", $limit_clause = "") {
		$sql = $this->VIEW_MODE_SQL." ".$this->WHERE_CLAUSE." ".$order_clause." ".$limit_clause;		
		return database_query($sql, _DATA_AND_ROWS);
	}
	
	/***
	 *	Get one record
	 **/
	function GetRecord($sql = "") {
		return database_query($sql, _DATA_AND_ROWS, _FIRST_ROW_ONLY);
	}
	
	/***
	 *	Add New record
	 **/
	function AddRecord() {
		
		//print_r($params);

		//----------------------------------------------------------------------
		// check if all required fields not empty
		foreach($this->arrAddModeFields  as $key => $val){
			if($val['required'] && $this->params[$key] == ""){
				$this->error = str_replace("_FIELD_", $val['title'], FIELD_CANNOT_BE_EMPTY);
				return false;
			}
			if(isset($this->params[$key]) && $this->params[$key] != "" && isset($val['validation_type'])){
				if(strtolower($val['validation_type']) == "email" && !$this->IsEmail($this->params[$key])){
					$this->error = str_replace("_FIELD_", $val['title'], FIELD_MUST_BE_EMAIL);
					return false;					
				}else if(strtolower($val['validation_type']) == "alpha" && !$this->IsAlpha($this->params[$key])){
					$this->error = str_replace("_FIELD_", $val['title'], FIELD_MUST_BE_ALPHA);
					return false;					
				}else if(strtolower($val['validation_type']) == "numeric" && !$this->IsNumeric($this->params[$key])){
					$this->error = str_replace("_FIELD_", $val['title'], FIELD_MUST_BE_NUMERIC);
					return false;					
				}else if(strtolower($val['validation_type']) == "alpha_numeric" && !$this->IsAlphaNumeric($this->params[$key])){
					$this->error = str_replace("_FIELD_", $val['title'], FIELD_MUST_BE_ALPHA_NUMERIC);
					return false;					
				}else if(strtolower($val['validation_type']) == "text" && !$this->IsText($this->params[$key])){
					$this->error = str_replace("_FIELD_", $val['title'], FIELD_MUST_BE_TEXT);
					return false;					
				}				
			}
		}

		//----------------------------------------------------------------------
		// block if this is a demo mode
		if(strtolower(_SITE_MODE) == "demo"){
			$this->error = OPERATION_BLOCKED;
			return false;				
		}
		
		//----------------------------------------------------------------------
		// insert
		$sql = "INSERT INTO ".$this->tableName."(";
			$sql .= $this->primaryKey;
			foreach($this->params as $key => $val){
				if(array_key_exists($key, $this->arrAddModeFields)){
					$sql .= ", ".$key;
				}
			}				
		$sql .= ") VALUES (";
			$sql .= "NULL";
			foreach($this->params as $key => $val){
				if(array_key_exists($key, $this->arrAddModeFields)){
					$sql .= ", '".$val."'";
				}
			}
		$sql .= ")";
		
		if(!database_void_query($sql, $this->debug)){
			$this->error = TRY_LATER;
			return false;
		}else{
			return true;
		}	
		
	}

	/***************************************************************************
	 *
	 *	UPDATE RECORD
	 *	
	 **************************************************************************/
	function UpdateRecord($rid = "0") {
		
		//print_r($params);

		//----------------------------------------------------------------------
		// check if all required fields not empty
		if($rid == "0" || $rid == "" || !is_numeric($rid)){
			$this->error = WRONG_PARAMETER_PASSED;
			return false;
		}		
		foreach($this->arrEditModeFields as $key => $val){
			$readonly = isset($val['readonly']) ? $val['readonly'] : false;
			if($val['required'] && $this->params[$key] == "" && !$readonly){				
				$this->error = str_replace("_FIELD_", $val['title'], FIELD_CANNOT_BE_EMPTY);
				return false;
			}
			if(isset($this->params[$key]) && $this->params[$key] != "" && isset($val['validation_type'])){
				if(strtolower($val['validation_type']) == "email" && !$this->IsEmail($this->params[$key])){
					$this->error = str_replace("_FIELD_", $val['title'], FIELD_MUST_BE_EMAIL);
					return false;					
				}else if(strtolower($val['validation_type']) == "alpha" && !$this->IsAlpha($this->params[$key])){
					$this->error = str_replace("_FIELD_", $val['title'], FIELD_MUST_BE_ALPHA);
					return false;					
				}else if(strtolower($val['validation_type']) == "numeric" && !$this->IsNumeric($this->params[$key])){
					$this->error = str_replace("_FIELD_", $val['title'], FIELD_MUST_BE_NUMERIC);
					return false;					
				}else if(strtolower($val['validation_type']) == "alpha_numeric" && !$this->IsAlphaNumeric($this->params[$key])){
					$this->error = str_replace("_FIELD_", $val['title'], FIELD_MUST_BE_ALPHA_NUMERIC);
					return false;					
				}else if(strtolower($val['validation_type']) == "text" && !$this->IsText($this->params[$key])){
					$this->error = str_replace("_FIELD_", $val['title'], FIELD_MUST_BE_TEXT);
					return false;					
				}				
			}			
		}

		//----------------------------------------------------------------------
		// block if this is a demo mode
		if(strtolower(_SITE_MODE) == "demo"){
			$this->error = OPERATION_BLOCKED;
			return false;				
		}
		
		//----------------------------------------------------------------------
		// update
		$sql = "UPDATE ".$this->tableName." SET ";
			$fields_count = 0;
			foreach($this->params as $key => $val){
				if(array_key_exists($key, $this->arrEditModeFields)){
                    if(isset($this->arrEditModeFields[$key]['readonly']) && $this->arrEditModeFields[$key]['readonly'] == true) continue;
					if($fields_count++ > 0) $sql .= ",";
					$sql .= $key." = '".$val."'";					
				}
			}				
		$sql .= " WHERE ".$this->primaryKey."=".(int)$rid;
		
		if(database_void_query($sql, $this->debug) < 0){
			$this->error = TRY_LATER;
			return false;
		}else{
			return true;
		}	
		
	}

	/***
	 *	Delete record
	 **/
	function DeleteRecord($rid = "")
	{
		//----------------------------------------------------------------------
		// check if rid is not empty
		if($rid == ""){				
			$this->error = WRONG_PARAMETER_PASSED;
			return false;
		}

		//----------------------------------------------------------------------
		// block if this is a demo mode
		if(strtolower(_SITE_MODE) == "demo"){
			$this->error = OPERATION_BLOCKED;
			return false;				
		}
		
		//----------------------------------------------------------------------
		// delete
		$sql = "DELETE FROM ".$this->tableName." WHERE ".$this->primaryKey." = ".(int)$rid."";
		if(database_void_query($sql, $this->debug) > 0){
			return true;
		}else{
			$this->error = TRY_LATER;
			return false;
		}
	}


	/***********************************************************************
	 *
	 *	Draw View Mode
	 *	
	 ***********************************************************************/
	function DrawViewMode()
	{
		global $text_alignment_left, $text_alignment_right;
		
		$concat_sign = (preg_match("/\?/", $this->formActionURL) ? "&" : "?");
		
		$arrRecords = $this->GetAll($this->ORDER_CLAUSE);
		if($this->allowLanguages) $arrLanguages = Languages::GetAll();		
		
		echo "<script type='text/javascript'>
		<!--
			function confirmDelete(rid){
				if(!confirm('".DELETE_WARNING_COMMON."')){
					false;
				}else{
					window.location.href = '".$this->formActionURL.$concat_sign."action=delete&rid='+rid+'&language_id=".$this->languageId."';
				}			
			}	
			function addNew(){
				window.location.href = '".$this->formActionURL.$concat_sign."&language_id=".$this->languageId."&action=add';
			}	
		//-->
		</script>";

		if($this->actions['add']){
			echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='main_text'>
				<tr><td nowrap height='7px'></td></tr>
				<tr>
					<td align='".$text_alignment_left."'><input class='form_button' type='button' name='btnAddNew' value=".ADD_NEW." onclick='javascript:addNew();'></td>
					<td align='".$text_alignment_right."'>
						".(($this->allowLanguages) ? get_languages_box("language_id", $arrLanguages[0], "abbreviation", "lang_name", $this->languageId, "", "onchange=\"window.location.href='".$this->formActionURL.$concat_sign."language_id='+this.value\"") : "")."
					</td>
				</tr>
				<tr><td nowrap height='5px'></td></tr>
			</table>";
		}

		if($arrRecords[1] > 0){			
			echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='main_text'>";
			echo "<tr>";
				foreach($this->arrViewModeFields as $key => $val){
					$width = isset($val['width']) ? "width='".$val['width']."'": "";
					$align = isset($val['align']) ? "align='".$val['align']."'": "";
					echo "<th ".$width." ".$align.">".$val['title']."</th>";
				}				
			echo "<th width='10%'>".ACTIONS."</th>";
			echo "</tr>";
			echo "<tr><td colspan='".(count($this->arrViewModeFields)+1)."' height='3px' nowrap='nowrap'>".draw_line("no_margin_line", _IMAGE_DIRECTORY, false)."</td></tr>";
			for($i=0; $i<$arrRecords[1]; $i++){
				echo "<tr ".highlight(0)." onmouseover='oldColor=this.style.backgroundColor;this.style.backgroundColor=\"#e1e1e1\";' onmouseout='this.style.backgroundColor=oldColor'>";
					foreach($this->arrViewModeFields as $key => $val){
						$align = isset($val['align']) ? "align='".$val['align']."'": "";
						echo "<td ".$align.">".$arrRecords[0][$i][$key]."</td>";
					}				
				echo "<td align='center' nowrap>
						<a href='".$this->formActionURL.$concat_sign."action=details&rid=".$arrRecords[0][$i]['id']."&language_id=".$this->languageId."'>".VIEW_WORD."</a>&nbsp;".get_divider()."&nbsp;";
						echo "<a href='".$this->formActionURL.$concat_sign."action=edit&rid=".$arrRecords[0][$i]['id']."&language_id=".$this->languageId."'>".EDIT_WORD."</a>";
				if($this->actions['delete']){ echo "&nbsp;".get_divider()."&nbsp;<a href='javascript:confirmDelete(\"".$arrRecords[0][$i]['id']."\");'>".DELETE_WORD."</a>"; }
				echo "&nbsp;</td></tr>";
			}
			echo "</table>";
		}else{
			echo draw_message(NO_RECORDS_FOUND);
		} 
	
	}

	/***********************************************************************
	 *
	 *	Draw Add Mode
	 *	
	 ***********************************************************************/
	function DrawAddMode(){
		
		echo "<form name='frmAddMode_".$this->tableName."' action='".$this->formActionURL."&language_id=".$this->languageId."' method='post'>\n";
		echo draw_hidden_field("action", "create")."\n";
        draw_token_field();

		// draw hidden fields
		foreach($this->arrAddModeFields as $key => $val){
			if($val['type'] == "hidden"){
				draw_hidden_field($key, $val['default'])."\n";
			}			
		}				
		
		// draw Add Form
		echo "\n<table width='100%' border='0' cellspacing='0' cellpadding='2' class='main_text'>\n";		
		echo "<tr><td colspan='2' height='5px' nowrap></td></tr>\n";
		foreach($this->arrAddModeFields as $key => $val){
			if($val['type'] != "hidden"){
				echo "<tr>";
				echo "  <td width='20%'>".$val['title'].(($val['required']) ? " <font color='#c13a3a'>*</font>" : "").":</td>";
				echo "  <td>".$this->DrawFieldByType($key, $val, $this->params, false)."</td>";
				echo "</tr>\n";				
			}			
		}				
		echo "<tr><td colspan='2' height='5px' nowrap></td></tr>\n";
		echo "<tr>\n
				<td colspan='2'>
					<input class='form_button' type='button' name='btnCancel' value=".BUTTON_CANCEL." onclick=\"javascript:window.location.href='".$this->formActionURL."&language_id=".$this->languageId."'\">
					<input class='form_button' type='submit' name='subAddNewRecord' value=".BUTTON_CREATE.">
				</td>
			  <tr>\n";		
		echo "</table><br />\n";
		echo "</form>\n";

		#<script type="text/javascript">
		#	mcms_SetFocus("header_text");
		#</script>	   

		
	}

	
	/***********************************************************************
	 *
	 *	Draw Edit Mode
	 *	
	 ***********************************************************************/
	function DrawEditMode($rid = "0"){
		
		echo "\n<form name='frmEditMode_".$this->tableName."' action='".$this->formActionURL."&language_id=".$this->languageId."' method='post'>\n";
		echo draw_hidden_field("action", "update")."\n";
		echo draw_hidden_field("rid", $rid)."\n";
        draw_token_field();
		
		// ger result for edited row		
		$this->EDIT_MODE_SQL = str_replace("_RID_", $rid, $this->EDIT_MODE_SQL);
		$result = database_query($this->EDIT_MODE_SQL, _DATA_AND_ROWS);
		
		// draw hidden fields
		foreach($this->arrEditModeFields as $key => $val){
			if($val['type'] == "hidden"){
				draw_hidden_field($key, $result[0][0][$key]);
				echo "\n";
			}			
		}				
		
		// draw Add Form
		echo "<table width='100%' border='0' cellspacing='0' cellpadding='2' class='main_text'>\n";
		echo "<tr><td colspan='2' height='5px' nowrap></td></tr>";
		foreach($this->arrEditModeFields as $key => $val){
			if($val['type'] != "hidden"){
				echo "<tr>\n";
				echo "  <td width='20%'>".$val['title'].((isset($val['required']) && $val['required']) ? " <font color='#c13a3a'>*</font>" : "").":</td>\n";
				echo "  <td>".(isset($result[0][0]) ? $this->DrawFieldByType($key, $val, $result[0][0], false) : '')."</td>\n";
				echo "</tr>\n";				
			}			
		}
		echo "<tr><td colspan='2' height='5px' nowrap></td></tr>";
		echo "<tr>
				<td colspan='2'>
					<input class='form_button' type='button' name='btnCancel' value=".BUTTON_CANCEL." onclick=\"javascript:window.location.href='".$this->formActionURL."&language_id=".$this->languageId."'\">
					<input class='form_button' type='submit' name='subUpdateRecord' value=".BUTTON_UPDATE.">
				</td>
			  <tr>\n";		
		echo "</table><br />\n";
		echo "</form>\n";

		#<script type="text/javascript">
		#	mcms_SetFocus("header_text");
		#</script>	   

		
	}


	
	/***********************************************************************
	 *
	 *	Draw Details Mode
	 *	
	 ***********************************************************************/
	function DrawDetailsMode($rid = "0"){
		
		// ger result for edited row		
		$this->DETAILS_MODE_SQL = str_replace("_RID_", $rid, $this->DETAILS_MODE_SQL);
		$result = database_query($this->DETAILS_MODE_SQL, _DATA_AND_ROWS);	
	
		// draw Add Form
		echo "<table width='100%' border='0' cellspacing='2' cellpadding='2' class='main_text'>\n";
		echo "<tr><td colspan='2' height='5px' nowrap></td></tr>";
		foreach($this->arrDetailsModeFields as $key => $val){
			if($val['type'] != "hidden"){
				echo "<tr>\n";
				echo "  <td width='25%'>".ucfirst($val['title']).":</td>\n";
				echo "  <td>".(isset($result[0][0]) ? $this->DrawFieldByType($key, $val, $result[0][0], false) : '')."</td>\n";
				echo "</tr>\n";				
			}			
		}
		echo "<tr><td colspan='2' height='5px' nowrap></td></tr>";
		echo "<tr>
				<td colspan='2'>
					<input class='form_button' type='button' name='btnBack' value=".BUTTON_BACK." onclick=\"javascript:window.location.href='".$this->formActionURL."&language_id=".$this->languageId."'\"> 
				</td>
			  <tr>\n";		
		echo "</table><br />\n";

	}


	
	function DrawFieldByType($field_name, $field_array = array(), $params = array(), $draw = true){
		$output = "";
		
		if($field_name == "") return false;

		$field_type   = isset($field_array['type']) ? $field_array['type'] : "";
		$source       = isset($field_array['source']) ? $field_array['source'] : "";
		$readonly     = (isset($field_array['readonly']) && $field_array['readonly'] !== "false") ? $field_array['readonly'] : false;

		$field_value  = isset($params[$field_name]) ? $params[$field_name] : "";
		if($this->isHtmlEncoding) $field_value = $this->GetDataDecoded($field_value);		
		
		switch($field_type){
			case "textarea":
				$output = "<textarea class='form_text' name='".$field_name."' id='".$field_name."' rows='7' cols='60'>".$field_value."</textarea>";			
				break;			

			case "label":
				$output = "<label class='form_label'>".$field_value."</label>";			
				break;			

			case "enum":
			    if(is_array($source)){				    
					$output_start = "<select name='".$field_name."'
									        id='".$field_name."'
							                ".(($readonly) ? "disabled='disabled'" : "").">";
					$output_options = "<option value=''>-- select --</option>";
					foreach($source as $key => $val){
						$output_options .= "<option value='".$key. "' ";
						$output_options .= ($field_value == $key) ? "selected='selected' " : "";
						$output_options .= ">".$val."</option>";
					}
					$output = $output_start.$output_options."</select>";					
				}
				break;			
			
			default:
			case "textbox":
				$output = "<input class='form_text' name='".$field_name."' id='".$field_name."' size='70' maxlength='255' value='".$field_value."' />";			
				break;			
			
		}
		
		if($draw) echo $output;
		else return $output;		
	}

	
	/***
	 * Get data encoded text
	 *		@param $string
	 **/
	private function GetDataEncodedText($string = ""){
		return $string;
	}
	
	
	/***
	 * Get data decoded text
	 *		@param $string
	 **/
	private function GetDataDecodedText($string = ""){
		$string = str_replace('\\\\', "\\", $string); // double quotation mark
		$string = str_replace('\"', '"', $string); // double quotation mark
		$string = str_replace("\'", "'", $string); // single quotation mark                        
		return $string;
	}
	
	
	/***
	 * Get data encoded
	 *		@param $string
	 **/
	private function GetDataEncoded($string = ""){
		return $string;
	}
	
	
	/***
	 * Get data decoded text
	 *		@param $string
	 **/
	private function GetDataDecoded($string = ""){
		$string = str_replace('\\\\', "\\", $string); // double quotation mark
		$string = str_replace('\"', "&#034;", $string); // double quotation mark
		$string = str_replace("\'", "&#039;", $string); // single quotation mark                        
		$string = str_replace('"', "&#034;", $string); // double quotation mark
		$string = str_replace("'", "&#039;", $string); // single quotation mark                        
		return $string;
	}

	////////////////////////////////////////////////////////////////////////////
	// Validation methods
	
	/***
	 * Email Validation
	 **/
	private function IsEmail($field = ""){
		$strict = false;
		$regex = $strict ? '/^([.0-9a-z_-]+)@(([0-9a-z-]+\.)+[0-9a-z]{2,4})$/i' :  '/^([*+!.&#$\'\\%\/0-9a-z^_`{}=?~:-]+)@(([0-9a-z-]+\.)+[0-9a-z]{2,4})$/i';
		
		if(preg_match($regex, trim($field))) {
		   return true;
		} else {
		   return false;
		}		
	}

	/***
	 * Numeric Validation
	 **/
	private function IsNumeric($field = ""){
		if(is_numeric($field)){
		   return true;
		} else {
		   return false;
		}		
	}

	/***
	 * Alpha Validation
	 **/
	private function IsAlpha($field = ""){
		if (ctype_alpha($field)){
		   return true;
		} else {
		   return false;
		}		
	}

	/***
	 * Alpha Numeric Validation
	 **/
	private function IsAlphaNumeric($field = ""){
		if(ctype_alnum($field)){
		   return true;
		} else {
		   return false;
		}		
	}

	/***
	 * Text Validation
	 **/
	private function IsText($field = ""){
		if(ctype_alnum($field) || $field != ""){
		   return true;
		} else {
		   return false;
		}		
	}
}
