<?php

/**
 *	Class ModulesSettings
 *  -------------- 
 *  Description : encapsulates Modules Settings operations & properties
 *  Updated	    : 25.11.2009
 *	Written by  : ApPHP
 *
 *	__construct
 *	__destruct
 *
 *	PUBLIC
 *	------------------
 *	UpdateRecord
 *	GetModuleSettings
 *	
 **/

class ModulesSettings extends MicroGrid {
	
	protected $tableName;
	protected $primaryKey;
	protected $dataSet;
	protected $formActionURL;
	protected $params;
	protected $actions;

	protected $VIEW_MODE_SQL;
	protected $EDIT_MODE_SQL;
	protected $DETAILS_MODE_SQL;

	protected $arrViewModeFields;	
	protected $arrAddModeFields;	
	protected $arrEditModeFields;
	protected $arrDetailsModeFields;
	
	public $error;
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct($module_name = "") {
		
		$this->params = array();		
		if(isset($_POST['settings_description'])) $this->params['settings_description'] = $_POST['settings_description'];
		if(isset($_POST['settings_key']))   	  $this->params['settings_key']   = $_POST['settings_key'];
		if(isset($_POST['settings_value']))       $this->params['settings_value'] = $_POST['settings_value'];
		if(isset($_POST['module_name']))          $this->params['module_name']    = $_POST['module_name'];
		if(isset($_POST['key_display_type']))     $this->params['key_display_type'] = $_POST['key_display_type'];

		$this->primaryKey 	= "id";
		$this->tableName 	= DB_PREFIX."modules_settings";
		$this->dataSet 		= array();
		$this->error 		= "";
		$this->formActionURL = "index.php?admin=".$module_name."_settings";
		$this->actions      = array("add"=>false, "edit"=>true, "details"=>true, "delete"=>false);


		//---------------------------------------------------------------------- 
		// VIEW MODE
		//---------------------------------------------------------------------- 
		$this->VIEW_MODE_SQL = "SELECT ".$this->primaryKey.",
									settings_key,
									settings_value,
									settings_description,
									key_display_type
								FROM ".$this->tableName."
								WHERE module_name = '".$module_name."'";		
		$this->arrViewModeFields = array(
			"settings_description" => array("title"=>NAME,  "type"=>"label", "width"=>"70%", "align"=>"left"),
			"settings_value"       => array("title"=>VALUE, "type"=>"label", "width"=>"", "align"=>"center")
		);
		
		//---------------------------------------------------------------------- 
		// ADD MODE
		//---------------------------------------------------------------------- 
		// define add mode fields
		$arr_key_display_types = array("string"=>"string", "numeric"=>"numeric", "enum"=>"enum");
		$this->arrAddModeFields = array(
			"settings_description" 	=> array("title"=>NAME,  "type"=>"textbox", "required"=>true),		
			"settings_key"   		=> array("title"=>KEY,   "type"=>"textbox", "required"=>true),
			"settings_value" 		=> array("title"=>VALUE, "type"=>"textbox", "required"=>true),			
			"key_display_type"      => array("title"=>KEY_DISPLAY_TYPE, "type"=>"enum", "source"=>$arr_key_display_types, "required"=>true),
			"module_name"    		=> array("title"=>"",    "type"=>"hidden",  "required"=>true, "default"=>$module_name),
		);

		//---------------------------------------------------------------------- 
		// EDIT MODE
		//---------------------------------------------------------------------- 
		$this->EDIT_MODE_SQL = "SELECT ".$this->primaryKey.",
								settings_key,
								settings_value,
								settings_description,
								key_display_type
							FROM ".$this->tableName."
							WHERE ".$this->primaryKey." = _RID_";		
		// define edit mode fields
		$this->arrEditModeFields = array(
			"settings_description" 	=> array("title"=>NAME,  "type"=>"textbox", "required"=>true),		
			"settings_value" 		=> array("title"=>VALUE, "type"=>"textbox", "required"=>true),
			"settings_key"   		=> array("title"=>KEY,   "type"=>"hidden"),			
		);

		//---------------------------------------------------------------------- 
		// DETAILS MODE
		//----------------------------------------------------------------------
		$this->DETAILS_MODE_SQL = $this->EDIT_MODE_SQL;
		$this->arrDetailsModeFields = array(
			"settings_description" 	=> array("title"=>NAME,  "type"=>"label"),
			"settings_value" 		=> array("title"=>VALUE, "type"=>"label"),
			"settings_key"   		=> array("title"=>KEY,   "type"=>"hidden"),			
		);

	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct()
	{
		// echo 'this object has been destroyed';
    }


	/***
	 *	Get all DataSet array
	 *
	 *  public static function GetAll() { }
	 *	
	 **/
	
	/***
	 *	Add record
	 *
	 *	function function AddRecord($params = array()) { 	}
	 *	
	 **/

	/***************************************************************************
	 *
	 *	UPDATE RECORD
	 *	
	 **************************************************************************/
	function UpdateRecord($rid = "0", $module_name = "") {

		//----------------------------------------------------------------------
		// check if all required fields not empty
		if($rid == "0" || $rid == "" || !is_numeric($rid)){
			$this->error = WRONG_PARAMETER_PASSED;
			return false;
		}
		
		foreach($this->arrEditModeFields as $key => $val){
			if($val['required']){
				if($this->params[$key] == ""){
					$this->error = str_replace("_FIELD_", $val['title'], FIELD_CANNOT_BE_EMPTY);
					return false;					
				}
			}
			if($key == "settings_key"){
				$sql = "SELECT key_display_type FROM ".$this->tableName."
					        WHERE settings_key = '".$this->params[$key]."' AND module_name = '".$module_name."'";					
				if($row = database_query($sql,_DATA_ONLY,_FIRST_ROW_ONLY)){
					if($row['key_display_type'] == "numeric" && !is_numeric($this->params['settings_value'])){
						$this->error = str_replace("_FIELD_", $val['title'], FIELD_MUST_BE_NUMERIC);
						return false;
					}else if($row['key_display_type'] == "yes/no" && (strtolower($this->params['settings_value']) != "yes" && strtolower($this->params['settings_value']) != "no")){
						$this->error = str_replace("_FIELD_", $val['title'], FIELD_MUST_BE_BOOLEAN);
						return false;						
					}
				}					
			}			
		}
		 
		//----------------------------------------------------------------------
		// block if this is a demo mode
		if(strtolower(_SITE_MODE) == "demo"){
			$this->error = OPERATION_BLOCKED;
			return false;				
		}
		
		//----------------------------------------------------------------------
		// update
		$sql = "UPDATE ".$this->tableName." SET ";
			$fields_count = 0;
			foreach($this->params as $key => $val){
				if(array_key_exists($key, $this->arrEditModeFields)){
					if($fields_count++ > 0) $sql .= ",";
					$sql .= $key." = '".$val."'";					
				}
			}				
		$sql .= " WHERE ".$this->primaryKey."=".(int)$rid;
		
		if(database_void_query($sql, $this->debug) < 0){
			$this->error = TRY_LATER;
			return false;
		}else{
			return true;
		}	
		
	}
	
	/***
	 *	Get module settings 
	 *
	 *		@param $module_name
	 *	
	 **/
	public function GetModuleSettings($module_name = "", $settings_key = "")
	{
		$sql = "SELECT settings_value as value
		        FROM ".$this->tableName."
				WHERE module_name = '".$module_name."'";
		if($settings_key != ""){
			$sql .= " AND settings_key = '".$settings_key."'";
			return database_query($sql, _DATA_ONLY, _FIRST_ROW_ONLY);			
		}else{
			return database_query($sql, _DATA_ONLY, _ALL_ROWS);			
		}
	}

	/***
	 *	Delete record
	 *
	 *	function DeleteRecord($rid = "") { 	}
	 *	
	 **/
	


}
?>