<?php

/***
 *	Class Settings
 *  ----------------- 
 *  Description : encapsulates site's settings and properties
 *  Updated	    : 24.05.2009
 *	Written by  : ApPHP
 *	
 **/

class Settings {

	public $error;
	
	private $res;
	private $template;
	private $header_text;
	private $footer_text;
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct(){
		$this->error = "";
		$this->template = "";
		$this->header_text = "";
		$this->footer_text = "";
		$this->test_direction = "";
		
		$sql = "SELECT * FROM ".DB_PREFIX."settings";
		$this->res = database_query($sql,_DATA_ONLY,_FIRST_ROW_ONLY);
		
		if($this->res){
			$this->header_text    = $this->res['header_text'];
			$this->footer_text    = $this->res['footer_text'];
			$this->template       = $this->res['template'];
			$this->text_direction = $this->res['text_direction'];
		}
	}

	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct(){
		// echo 'this object has been destroyed';
    }


	/***
	 *	Set text direction
	 *		@param $text_direction - ltr|rtl
	 **/
	public function SetDirection($text_direction = "ltr"){
		$sql = "UPDATE ".DB_PREFIX."settings SET text_direction = '".$text_direction."'";
		if (database_void_query($sql)){
			$this->template = $template;
			return true;
		} else {
			$this->error = TRY_LATER;
			return false;
		}							
	}
	
	/***
	 *	Get current text direction
	 *		
	 **/
	public function GetDirection(){
		return $this->text_direction;
	}

	/***
	 *	Get template name
	 *		
	 **/
	public function GetTemplate(){
		return $this->GetDataDecoded($this->template);
	}
	
	/***
	 *	Get parameter value by name
	 *		@param $field_name	
	 **/
	public function GetParameter($field_name = ""){
		if(isset($this->res[$field_name])){
			return $this->GetDataDecoded($this->res[$field_name]);	
		}else{
			return "";
		}
	}

	/***
	 *	Update fields
	 *		@param $params - pairs: field - value 
	 **/
	public function UpdateFields($params = array()){
		
		if(count($params) > 0){
			if(strtolower(_SITE_MODE) == "demo"){
				$this->error = OPERATION_BLOCKED;
				return false;				
			}else{
				// prepare UPDATE statement
				$sql = "UPDATE ".DB_PREFIX."settings SET ";
				$count = 0;
				foreach($params as $key => $val){
					if($count++ > 0) $sql .= ", ";
					$sql .= $key." = '".$val."'";				
				}
				if (database_void_query($sql)){
					return true;
				} else {
					$this->error = TRY_LATER;
					return false;
				}				
			}
		}else{
			return "";						
		}
	}
	

	/***
	 *	Set site template
	 *		@param $template - template name
	 **/
	public function SetTemplate($template = ""){
		$sql = "UPDATE ".DB_PREFIX."settings SET template = '".$this->GetDataEncoded($template)."'";
		if (database_void_query($sql)){
			$this->template = $template;
			return true;
		} else {
			$this->error = TRY_LATER;
			return false;
		}							
	}
	
	/***
	 *	Set text direction
	 *		@param $text_direction - ltr|rtl
	 **/
	public function GetHeader(){
		return $this->GetDataDecoded($this->header_text);
	}

	/***
	 *	Set site header
	 *		@param $header_text
	 **/
	public function SetHeader($header_text = ""){
		if(strtolower(_SITE_MODE) == "demo"){
			$this->error = OPERATION_BLOCKED;
			return false;				
		}else{
			$sql = "UPDATE ".DB_PREFIX."settings SET header_text = '".$this->GetDataEncoded($header_text)."'";
			if(database_void_query($sql)){
				$this->header_text = $header_text;
				return true;
			} else {
				$this->error = TRY_LATER;
				return false;
			}							
		}
	}
	
	/***
	 *	Draw site's footer
	 *		
	 **/
	public function DrawFooter(){
		echo $this->GetDataTagged($this->footer_text);
	}

	/***
	 *	Get site's footer text
	 *		
	 **/
	public function GetFooter(){
		return $this->GetDataDecoded($this->footer_text);
	}

	/***
	 *	Set site's footer text
	 *		@param $footer_text
	 **/
	public function SetFooter($footer_text = ""){
		if(strtolower(_SITE_MODE) == "demo"){
			$this->error = OPERATION_BLOCKED;
			return false;				
		}else{
			$sql = "UPDATE ".DB_PREFIX."settings SET footer_text = '".$this->GetDataEncoded($footer_text)."'";
			if (database_void_query($sql)){
				$this->footer_text = $footer_text;
				return true;
			} else {
				$this->error = TRY_LATER;
				return false;
			}							
		}
	}

	/***
	 *	Get encoded text
	 *		@param $string
	 **/
	private function GetDataEncodedText($string = ""){
		return $string;
	}
	
	/***
	 *	Get decoded text 
	 *		@param $string
	 **/
	private function GetDataDecodedText($string = ""){
		$string = str_replace('\"', '"', $string); 		// double quotation mark
		$string = str_replace("\'", "'", $string); 		// single quotation mark                        
		return $string;
	}
	
	/***
	 *	Get encoded string
	 *		@param $string
	 **/
	private function GetDataEncoded($string = ""){
		return $string;
	}
	
	/***
	 *	Get decoded string
	 *		@param $string
	 **/
	private function GetDataDecoded($string = ""){
		$string = str_replace('\\\\', "\\", $string);   
		$string = str_replace('\"', "&#034;", $string); // double quotation mark
		$string = str_replace("\'", "&#039;", $string); // single quotation mark                        
		$string = str_replace('"', "&#034;", $string);  // double quotation mark
		$string = str_replace("'", "&#039;", $string);  // single quotation mark                        
		return $string;
	}
	
	/***
	 *	Get tagged string 
	 *		@param $string
	 **/
	function GetDataTagged($string){
		$string = str_replace("&lt;", "<", $string); 
		$string = str_replace("&gt;", ">", $string); 
		return $string;		
	}
	
}
?>