<?php

/***
 *	Class Vocabulary
 *  -------------- 
 *  Description : encapsulates vocabulary properties and methods
 *  Updated	    : 20.05.2009
 *	Written by  : ApPHP
 *	
 **/

class Vocabulary {

	public $error;
	
	protected $keys;
	protected $filterBy;
	protected $filterByUrl;
	protected $languageId;
	protected $langIdByUrl;
	protected $whereClause;
	protected $isKeyUpdated;
	protected $currentKey;

	private $vocabularySize;
	
	//==========================================================================
    // Class Constructor
	//==========================================================================
	function __construct() {
		// get filter value
		$this->filterBy = isset($_REQUEST['filter_by']) ? htmlentities($_REQUEST['filter_by']) : "";
		$this->filterByUrl = ($this->filterBy != "") ? "&filter_by=".$this->filterBy : "";

		$this->languageId  = (isset($_REQUEST['language_id']) && $_REQUEST['language_id'] != "") ? htmlentities($_REQUEST['language_id']) : Languages::GetDefaultLang();
		$this->langIdByUrl = ($this->languageId != "") ? "&language_id=".$this->languageId : "";

		$this->whereClause  = "";
		$this->whereClause .= ($this->languageId != "") ? " AND language_id = '".$this->languageId."'" : "";		
		$this->whereClause .= ($this->filterBy != "") ? " AND key_value LIKE '".$this->filterBy."%'" : "";		
		
		$this->isKeyUpdated = false;
		$this->vocabularySize = 0;
		$this->currentKey = "";
	}
	
	//==========================================================================
    // Class Destructor
	//==========================================================================
    function __destruct(){
		// echo 'this object has been destroyed';
    }

	/***
	 * Return filter URL
	 *		
	 **/
	public function GetFilterURL(){
		return $this->filterByUrl;		
	}

	/***
	 * Return language URL
	 *		
	 **/
	public function GetLanguageURL(){
		return $this->langIdByUrl;		
	}


	/***
	 * Return vocabulary record set
	 *		@param $where_clause
	 **/
	private function GetVocabulary($where_clause = ""){
		$sql = "SELECT * FROM ".DB_PREFIX."vocabulary WHERE 1=1 ".$where_clause." ORDER BY key_value ASC";
		$this->keys = database_query($sql, _DATA_ONLY, _ALL_ROWS);
		$this->vocabularySize = count($this->keys);
	}
	
	/***
	 * Return vocabulary size
	 *		@param $where_clause
	 **/
	private function GetVocabularySize($where_clause = ""){
		$sql = "SELECT COUNT(*) FROM ".DB_PREFIX."vocabulary";
		$this->vocabularySize = database_query($sql, _ROWS_ONLY);		
	}

	/***
	 * Draw rewrite button
	 *		
	 **/
	public function DrawRewriteButton(){
		global $objSettings;
		$total_languages = Languages::GetAll();
		
		$button_align_left  = ($objSettings->GetDirection() == "ltr") ? "text-align: left;" : "text-align: right;";
		$button_align_right = ($objSettings->GetDirection() == "ltr") ? "text-align: right;" : "text-align: left;";

		if($this->GetVocabularySize() <= 0){			
			echo "<form action='index.php?admin=vocabulary' method='post'>";
			draw_hidden_field("submition_type", "2");
            draw_token_field();
			echo "<table align='center' width='98%' border='0' cellspacing='0' cellpadding='3' class='main_text'>
				  <tr valign='top' >					
					<td style='".$button_align_left."'>
						".get_languages_box("language_id", $total_languages[0], "abbreviation", "lang_name", $this->languageId, "", "onchange=\"window.location.href='index.php?admin=vocabulary".$this->filterByUrl."&language_id='+this.value\"")."
					</td>
					<td style='".$button_align_right."'>
						<input class='form_button' type='submit' name='submit' value='".$this->GetDataDecoded(BUTTON_REWRITE)."'>
					</td>
				  </tr>
				  </table>";
			echo "</form>";			
		}
	}	
	
	/***
	 * Draw edit form
	 *		@param $key 
	 **/
	public function DrawEditForm($key = "0")
	{
		$total_languages = Languages::GetAll();	
		$key_value = $key_text = "";
		
		$sql = "SELECT * FROM ".DB_PREFIX."vocabulary WHERE id = ".(int)$key."";
        if($row = database_query($sql,_DATA_ONLY,_FIRST_ROW_ONLY)){
			$key_value = $row['key_value'];
			$key_text = $row['key_text'];
			$this->currentKey = $key_value;
		}
		echo "<form action='index.php?admin=vocabulary' method='post'>";
				draw_hidden_field("submition_type", "1");
				draw_hidden_field("key", $key);
				draw_hidden_field("filter_by", $this->filterBy);
				draw_hidden_field("language_id", $this->languageId);
                draw_token_field();
		  echo "<table align='center' width='98%' border='0' cellspacing='0' cellpadding='3' class='main_text'>
				<tr valign='top'>
					<td align='left'><b>".EDIT_WORD."</b></td>
					<td></td>
					<td align='right'>".get_languages_box("language_id", $total_languages[0], "abbreviation", "lang_name", $this->languageId, "", "disabled='disabled'")."</td>
				</tr>
				<tr valign='top'>
					<td align='right' width='110px'>".KEY.":</td>
					<td align='left'>
						".$key_value."
						<input type='hidden' name='txt_key' value='".$key_value."' />
					</td>
					<td></td>
				</tr>
				<tr valign='top'>
					<td align='right'>".VALUE." <font color='#c13a3a'>*</font>:</td>
					<td align='left'>
						<input type='text' style='width:360px;' name='txt_key_value' value='".$this->GetDataDecoded($key_text)."' />
					</td>
					<td align='right'>
						<input class='form_button' type='submit' name='submit' value='".$this->GetDataDecoded(BUTTON_UPDATE)."'>&nbsp;
						<input class='form_button' type='button' onclick=\"window.location.href='index.php?admin=vocabulary".$this->langIdByUrl."'\" value='".$this->GetDataDecoded(BUTTON_CANCEL)."'>					
					</td>
				</tr>
				<tr align='right'>
					<td colspan='3'></td>
				</tr>
				</table>
			 </form>";			 
	}
	
	
	/***
	 * Check if key was updated
	 *		
	 **/
	public function IsKeyUpdated(){
		return $this->isKeyUpdated;
	}

	/***
	 * Update vocabulary key
	 *		@param $key_value
	 *		@param $key_text
	 **/
	public function UpdateKey($key_value = "", $key_text = ""){
		
		// Block all operations in demo mode
		if(strtolower(_SITE_MODE) == "demo"){
			$this->error = OPERATION_BLOCKED;
			return false;				
		}

		// Check input parameters
		if($key_text == ""){
			$this->error = VOC_KEY_VALUE_EMPTY;
			return false;
		}
		
		$sql = "UPDATE ".DB_PREFIX."vocabulary
				SET key_text = '".$this->GetDataEncoded($key_text)."'
				WHERE
					key_value = '".$key_value."' AND
					language_id = '".$this->languageId."'";
		if(database_void_query($sql) >= 0){
			$this->isKeyUpdated = true;
			return true;
		}else{
			$this->error = TRY_LATER;
			return false;
		}            
	}
			
			
	/***
	 * Draw vocabulary
	 *		
	 **/
	public function DrawVocabulary(){
		
		$this->GetVocabulary($this->whereClause);
			
		echo "<table width='98%' align='center' border='0' cellspacing='0' cellpadding='2' class='main_text'>
			  <tr>
				<td>".FILTER_BY.": ";
				echo "<a href='index.php?admin=vocabulary".$this->langIdByUrl."'>".ALL."</a> - ";
				for($i=65; $i<91; $i++){
					if($this->filterBy == chr($i)) $chr_i = "<b><u>".chr($i)."</u></b>";
					else $chr_i = chr($i);
					echo "<a href='index.php?admin=vocabulary&filter_by=".chr($i).$this->langIdByUrl."'>".$chr_i."</a> ";
				}
				echo "</td>
			  </tr>";
		echo "<tr align='center'><td>".draw_line("line_no_margin", _IMAGE_DIRECTORY, false)."</td></tr>";	
		echo "</table>";

		if(!empty($this->keys)){							
           
            echo "<table width='98%' align='center' border='0' cellspacing='0' cellpadding='3' class='main_text'>";
			echo "<tr>
					<th width='30%' align='left'>".KEY."</th>
					<th width='60%' align='left'>".VALUE."</th>
					<th width='10%'></th>";

			for($i=0; $i < $this->vocabularySize; $i++){

				// Prepare key_text for displaying
				$decoded_text = strip_tags($this->GetDataDecoded($this->keys[$i]['key_text']));
				if(strlen($decoded_text) > 54){
					$key_text = "<span style='cursor:help;' title='".$decoded_text."'>".substr($decoded_text,0,54)."...</span>";
				}else{
					$key_text = $decoded_text;
				}

				// Display vocabulary row
				if($this->keys[$i]['key_value'] == $this->currentKey){
					echo "<tr>";
					echo "<td align='left' style='background-color:#f3f3d2; border:0px; border-left:1px solid #c2c23b; border-top:1px solid #c1c13a; border-bottom:1px solid #c1c13a;'>".$this->keys[$i]['key_value']."</td>";
					echo "<td align='left' style='background-color:#f3f3d2; border:0px; border-top:1px solid #c2c23b; border-bottom:1px solid #c1c13a;'>".$key_text."</td>
					      <td align='center' style='background-color:#f3f3d2; border:0px; border-right:1px solid #c2c23b; border-top:1px solid #c1c13a; border-bottom:1px solid #c1c13a;'><a href='index.php?admin=vocabulary&key=".$this->keys[$i]['id']."&act=edit".$this->filterByUrl.$this->langIdByUrl."'>[".EDIT_WORD."]</a></td>
					</tr>";				
				}else{
					echo "<tr ".highlight(0)." onmouseover='oldColor=this.style.backgroundColor;this.style.backgroundColor=\"#ededed\";' onmouseout='this.style.backgroundColor=oldColor'>";
					echo "<td align='left'>".$this->keys[$i]['key_value']."</td>";
					echo "<td align='left'>".$key_text."</td>
					  <td align='center'><a href='index.php?admin=vocabulary&key=".$this->keys[$i]['id']."&act=edit".$this->filterByUrl.$this->langIdByUrl."'>[".EDIT_WORD."]</a></td>
					</tr>";				
				}
			}			
			echo "</table>";
		}else{
			draw_important_message(VOC_NOT_FOUND);
		}	
		
	}


	/***
	 * Rewrite vocabulary file
	 *		
	 **/
	public function RewriteVocabularyFile($all_languages = false){
	
		// Block all operations in demo mode
		if(strtolower(_SITE_MODE) == "demo"){
			$this->error = OPERATION_BLOCKED;
			return false;				
		}

		$this->GetVocabulary((!$all_languages) ? "AND language_id = '".$this->languageId."'" : "");
		
		$string_data ="<?php\n";		
		for($i=0; $i < $this->vocabularySize; $i++){			
			$string_data .= 'define("'.$this->keys[$i]['key_value'].'","'.$this->keys[$i]['key_text'].'");'."\n";
		}		
		$string_data .= "\n?>";
		
		// Write data to the file
		if(!$all_languages){
			$voc_file ="include/messages.".$this->languageId.".inc.php";			
		}else{
			$voc_file ="include/messages.inc.php";
		}
		@chmod($voc_file, 0755);
		$fh = fopen($voc_file, 'w');
		if(!$fh){
			$this->error = "Can not open vocabulary file: ".$voc_file;
		}else{
			fwrite($fh, $string_data);
			fclose($fh);						
		}
		@chmod($voc_file, 0644);
		
		return true;	
	}
	

	/***
	 * Get data encoded text
	 *		@param $string
	 **/
	private function GetDataEncodedText($string = ""){
		return $string;
	}
	
	
	/***
	 * Get data decoded text
	 *		@param $string
	 **/
	private function GetDataDecodedText($string = ""){
		$string = str_replace('\\\\', "\\", $string); // double quotation mark
		$string = str_replace('\"', '"', $string); // double quotation mark
		$string = str_replace("\'", "'", $string); // single quotation mark                        
		return $string;
	}
	
	
	/***
	 * Get data encoded
	 *		@param $string
	 **/
	private function GetDataEncoded($string = ""){
		$string = str_replace('"', '\"', $string); // double quotation mark
		return $string;
	}
	
	
	/***
	 * Get data decoded
	 *		@param $string
	 **/
	private function GetDataDecoded($string = ""){
		$string = str_replace('\\\\', "\\", $string); // double quotation mark
		$string = str_replace('\"', "&#034;", $string); // double quotation mark
		$string = str_replace("\'", "&#039;", $string); // single quotation mark                        
		$string = str_replace('"', "&#034;", $string); // double quotation mark
		$string = str_replace("'", "&#039;", $string); // single quotation mark                        
		return $string;
	}
}
?>